@@ -4,6 +4,7 @@ import locale
 import io
 import csv
 import curses
+import hashlib
 from os import rename, remove, access, X_OK, getenv, makedirs
 from os.path import exists, dirname, join, expanduser
 from shutil import which, move, Error as shutil_Error
@@ -63,6 +64,7 @@ M_STRINGS = {
 	'press-?': ' Press ? for help',
 	'error-str': 'error',
 	'vol_': 'Vol: ',
+    'volume_set': 'Volume set to ',
     'error-403': 'Server returned "Forbidden" (error 403)',
     'error-404': 'Station does not exist (error 404)',
     'error-503': 'Service not available (error 503)',
@@ -405,16 +407,6 @@ class StationsChanges():
         self.last_sync = None
         self.version_to_write = None
 
-    def _read_version(self):
-        the_file = join(dirname(__file__), '__init__.py')
-        lin = ''
-        with open(the_file, 'r', encoding='utf-8') as cfg:
-            while not lin.startswith('version_info'):
-                lin = cfg.readline().strip()
-        lin = lin[15:].replace('(', '').replace(')', '')
-        # this_version = tuple(map(int, lin.split(', ')))
-        return eval(lin)
-
     def _read_synced_version(self, asked=False):
         in_file = self._asked_sync_file if asked else self._last_sync_file
         # print('in_file = "{}"'.format(in_file))
@@ -473,7 +465,7 @@ class StationsChanges():
                 writter = csv.writer(cfgfile)
                 for a_station in self._stations:
                     if a_station[3] != '':
-                        a_station[3] = a_station[3]['image']
+                        a_station[3] = a_station[3]
                     writter.writerow(self._format_playlist_row_out(a_station))
         except:
             print('Error: Cannot create the updated stations file.')
@@ -797,7 +789,7 @@ class CsvReadWrite():
 
                         # Append the parsed values to the reading stations list
                         station_info = [
-                            name, url, enc, {'image': icon} if icon else '',
+                            name, url, enc, icon if icon else '',
                             profile, buffering, http, volume, referer, player
                         ]
                         self._items.append(station_info)
@@ -829,9 +821,6 @@ class CsvReadWrite():
                 Eliminates any trailing empty fields
         '''
         this_row = a_row[:]
-        # Extract the 'image' from the icon dictionary if present
-        if len(this_row) > Station.icon and 'image' in this_row[Station.icon]:
-            this_row[Station.icon] = this_row[Station.icon]['image']
         if self.encoding_to_remove is not None:
             if this_row[Station.encoding] == self.encoding_to_remove:
                 this_row[Station.encoding] = ''
@@ -1273,3 +1262,41 @@ def validate_resource_opener_path(a_file):
     # Return the validated path
     return a_file
 
+def get_cached_icon_path(cache_dir, station_name, icon_url):
+    """
+    Get cached icon path, migrating from station-based to URL-based naming
+    """
+    # Old filename (based on station name)
+    old_filename = f"{station_name.replace(' ', '_')}."
+    ext = 'jpg' if icon_url.endswith(('.jpg', '.jpeg')) else 'png'
+    path1 = join(cache_dir, old_filename + ext)
+
+    # New filename (based on icon URL hash)
+    url_hash = hashlib.md5(icon_url.encode()).hexdigest()
+    new_filename = f"{url_hash}.{ext}"
+    path2 = join(cache_dir, new_filename)
+
+    # Migration logic
+    if exists(path1):
+        if exists(path2):
+            # Both new and old filenames exist - delete old
+            try:
+                remove(path1)
+            except OSError as e:
+                # Log the error but continue with path2
+                if logger.isEnabledFor(logging.DEBUG):
+                    logger.debug(f'Failed to remove old icon {path1}: {e}')
+            return path2
+        else:
+            # Old filename exists - rename to new
+            try:
+                rename(path1, path2)
+            except OSError as e:
+                # If rename fails, fall back to old path
+                if logger.isEnabledFor(logging.DEBUG):
+                    logger.debug(f'Failed to rename {path1} to {path2}: {e}')
+                return path1
+            return path2
+    else:
+        # Old filename does not exist - use the new one
+        return path2
