@@ -12,6 +12,10 @@ import curses
 from os.path import join, exists, dirname
 from collections import deque
 from threading import Lock
+try:
+    from importlib.resources import files, as_file   # 3.9+
+except ImportError:
+    from importlib_resources import files, as_file   # backport για 3.7–3.8
 try:
     from .cjkwrap import is_wide
 except ImportError:
@@ -27,7 +31,7 @@ queue_lock = Lock()
 def enqueue_input(char, reset=False):
     global input_queue
     if reset:
-        input_queue(clear)
+        input_queue.clear()
     with queue_lock:
         input_queue.append(char)
 
@@ -149,13 +153,14 @@ kbkey_orig['change_player']            = ( ord('m')               , 'Cahnge medi
 kbkey_orig['hist_top']                 = ( ord(']')               , 'Open first opened playlist')
 kbkey_orig['buffer']                   = ( ord('b')               , 'Toggle buffering')
 kbkey_orig['toggle_time']              = ( ord('t')               , 'Toggle displaying time')
+kbkey_orig['toggle_tts']               = ( ord('T')               , 'Toggle TTS')
 kbkey_orig['open_buffer']              = ( ord('B')               , 'Open buffering window')
 kbkey_orig['last_playlist']            = ( ord('l')               , 'Toggle Open last playlist')
 kbkey_orig['clear_reg']                = ( ord('c')               , 'Clear current register')
 kbkey_orig['clear_all_reg']            = ( ord('C')               , 'Clear all registers')
 kbkey_orig['unnamed']                  = ( ord('u')               , 'Show unnamed register')
-kbkey_orig['html_help']                = ( ord('h')               , 'Open html help')
-kbkey_orig['station_volume']           = ( ord('v')               , 'Set station volume')
+kbkey_orig['html_help']                = ( ord('h')               , 'Display html help')
+kbkey_orig['station_volume']           = ( ord('v')               , 'Save station volume')
 kbkey_orig['toggle_station_volume']    = ( ord('V')               , 'Toggle use of station volume')
 
 # ! RadioBrowser Keys:
@@ -221,29 +226,54 @@ def check_localized(char, k_list, return_key=False):
     # logger.error('\n\n')
     # logger.error(f'{k_list = }')
     # logger.error(f'{kb_letter = }')
-    if char and kb_letter and isinstance(k_list, Iterable) and lkbkey is not None:
-        # logger.error(f'{char = }')
-        # logger.error(f'{chr(char) = }')
-        for n in k_list:
-            # logger.error(f'k_list: {n = }')
-            # logger.error(f'k_list: {chr(n) = }')
-            # logger.error(f'*** { kb_letter =  }')
-            # logger.error(f'*** { lkbkey =  }')
-            if kb_letter in lkbkey:
-                x = lkbkey[kb_letter]
-                # logger.error(f'{x = }')
-                if x == chr(n):
-                    if logger.isEnabledFor(logging.DEBUG):
-                        logger.debug('localized char found: "{0}" => "{1}"'.format(kb_letter, x))
-                    # logger.error('\n\n')
-                    if return_key:
-                        # logger.info(f'returning {n}')
-                        return n
-                    # logger.info('returning True')
-                    return True
-            else:
-                # logger.error('kb_letter not in lkbkey')
-                continue
+    # logger.error(f'{lkbkey = }')
+    if platform.system().lower().startswith('win'):
+        if char and isinstance(k_list, Iterable) and lkbkey is not None:
+            # logger.error(f'{char = }')
+            # logger.error(f'{chr(char) = }')
+            for n in k_list:
+                # logger.error(f'k_list: {n = }')
+                # logger.error(f'*** { kb_letter =  }')
+                # logger.error(f'*** { lkbkey =  }')
+                key = chr(char)
+                # logger.error(f'*** { key =  }')
+                if char in lkbkey:
+                    if ord(lkbkey[char]) == n:
+                        if logger.isEnabledFor(logging.DEBUG):
+                            logger.debug('localized char found: {0} => "{1}"'.format(char, lkbkey[char]))
+                        # logger.error('\n\n')
+                        if return_key:
+                            # logger.info(f'returning {n}')
+                            return n
+                        # logger.info('returning True')
+                        return True
+                else:
+                    # logger.error('char not in lkbkey')
+                    continue
+    else:
+        if char and kb_letter and isinstance(k_list, Iterable) and lkbkey is not None:
+            # logger.error(f'{char = }')
+            # logger.error(f'{chr(char) = }')
+            for n in k_list:
+                # logger.error(f'k_list: {n = }')
+                # logger.error(f'k_list: {chr(n) = }')
+                # logger.error(f'*** { kb_letter =  }')
+                # logger.error(f'*** { lkbkey =  }')
+                if kb_letter in lkbkey:
+                    x = lkbkey[kb_letter]
+                    # logger.error(f'{x = }')
+                    if x == chr(n):
+                        if logger.isEnabledFor(logging.DEBUG):
+                            logger.debug('localized char found: "{0}" => "{1}"'.format(kb_letter, x))
+                        # logger.error('\n\n')
+                        if return_key:
+                            # logger.info(f'returning {n}')
+                            return n
+                        # logger.info('returning True')
+                        return True
+                else:
+                    # logger.error('kb_letter not in lkbkey')
+                    continue
     # logger.error('\n\n')
     if return_key:
         # logger.info('returning None')
@@ -351,21 +381,23 @@ def read_localized_keyboard(localize, data_dir):
         error = True
     else:
         user_file = join(data_dir, 'lkb_' + localize + '.json')
-        package_file = join(dirname(__file__), 'keyboard', 'lkb_' + localize + '.json')
-        target_file = None
-        if exists(package_file):
-            target_file = package_file
-        if exists(user_file):
-            target_file = user_file
+        package_res = files("pyradio").joinpath("keyboard", 'lkb_' + localize + '.json')
 
-        if target_file is None:
-            error = True
-        else:
+        if exists(user_file):
             try:
-                with open(target_file, 'r', encoding='utf-8', errors='ignore') as json_file:
+                with open(user_file, 'r', encoding='utf-8', errors='ignore') as json_file:
                     data = json.load(json_file)
-            except (FileNotFoundError, json.JSONDecodeError, TypeError, IOError):
+            except (json.JSONDecodeError, TypeError, IOError):
                 error = True
+        elif package_res.is_file():  # package resource
+            with as_file(package_res) as tmp_path:
+                try:
+                    with open(tmp_path, 'r', encoding='utf-8', errors='ignore') as json_file:
+                        data = json.load(json_file)
+                except (json.JSONDecodeError, TypeError, IOError):
+                    error = True
+        else:
+            error = True
 
     if error:
         # keys = list(string.ascii_lowercase) + list(string.ascii_uppercase)
@@ -375,7 +407,10 @@ def read_localized_keyboard(localize, data_dir):
         return
 
     # Reverse the keys and values
-    reversed_dict = {value: key for key, value in data.items()}
+    if platform.system().lower().startswith('win'):
+        reversed_dict = {ord(value): key for key, value in data.items()}
+    else:
+        reversed_dict = {value: key for key, value in data.items()}
 
     # logger.error('\n\nsetting lkbkey 2\n{}\n\n'.format(reversed_dict))
     set_lkbkey(reversed_dict)
@@ -630,12 +665,15 @@ def chk_key(char, key, win):
 
 def set_kb_letter(letter):
     global kb_letter
-    if letter and letter.isprintable():  # Check if the letter is printable
-        kb_letter = letter
-        # logger.error(f'>>> {kb_letter = }')  # Log the printable letter
-    else:
-        kb_letter = ''  # Ignore non-printable characters
-        # logger.error(f'>>> Ignored non-printable letter: {repr(letter)}')
+    try:
+        if letter and letter.isprintable():  # Check if the letter is printable
+            kb_letter = letter
+            # logger.error(f'>>> {kb_letter = }')  # Log the printable letter
+        else:
+            kb_letter = ''  # Ignore non-printable characters
+            # logger.error(f'>>> Ignored non-printable letter: {repr(letter)}')
+    except AttributeError:
+        kb_letter = ''
     if logger.isEnabledFor(logging.DEBUG) and kb_letter:
         logger.debug(f'setting {kb_letter = }')
 
@@ -730,15 +768,15 @@ def add_l10n_to_functions_dict(functions):
     else:
         local_functions = functions.copy()
         new_keys = {}
-        for key, value in local_functions.items():
-            logger.error(f'{key}: {value}')
-            for lkey, lvalue in local_keys.items():
-                if ord(lvalue) == key:
-                    logger.error(f'    {lkey}: {lvalue}')
-                    logger.error('      {} {}'.format(ord(lvalue), value))
-                    new_keys[ord(lvalue)] = value
-                    break
-        logger.error('\n\n{}'.format(new_keys))
+        # for key, value in local_functions.items():
+        #     logger.error(f'{key}: {value}')
+        #     for lkey, lvalue in local_keys.items():
+        #         if ord(lvalue) == key:
+        #             logger.error(f'    {lkey}: {lvalue}')
+        #             logger.error('      {} {}'.format(ord(lvalue), value))
+        #             new_keys[ord(lvalue)] = value
+        #             break
+        # logger.error('\n\n{}'.format(new_keys))
         for n in new_keys:
             local_functions[n] = new_keys[n]
     return local_functions
