@@ -8,12 +8,19 @@ from sys import platform, stdout
 from platform import system as platform_system
 from copy import deepcopy
 from time import sleep
+from shutil import copy2
+from pathlib import Path
 import datetime
 import logging
 import threading
 import subprocess
+try:
+    from importlib.resources import files, as_file   # 3.9+
+except ImportError:
+    from importlib_resources import files, as_file   # backport για 3.7–3.8
 from .common import STATES, M_STRINGS
-from .cjkwrap import cjklen
+from .cjkwrap import cjklen, cjkslices
+from .tts import Priority
 
 locale.setlocale(locale.LC_ALL, "")
 
@@ -240,7 +247,9 @@ class Log():
                  config,
                  current_player_id,
                  active_player_id,
-                 get_web_song_title
+                 get_web_song_title,
+                 tts,
+                 mode
                  ):
         self.program_restart = False
         self.error_msg = False
@@ -248,6 +257,7 @@ class Log():
         self._desktop_notification_title = None
         self._desktop_notification_message = None
         self._repeat_notification = None
+        self._mode = mode
 
         self.msg = None
         self.suffix = None
@@ -308,6 +318,7 @@ class Log():
         self._stop_thread = False
         self.timer = None
         self._started_station_name = None
+        self.tts = tts
 
     def __del__(self):
         self._stop_desktop_notification_thread = True
@@ -405,6 +416,23 @@ class Log():
               error_msg=False,
               notify_function=None):
         with self.lock:
+            try:
+                if self.tts and self.tts():
+                    if msg_id == STATES.TITLE \
+                            and msg and msg.startswith(M_STRINGS['title_']):
+                        self.tts().queue_speech(msg, Priority.HIGH, mode=self._mode())
+                    elif msg_id == STATES.VOLUME:
+                        if msg.startswith('['):
+                            pass
+                            # logger.error(f'{msg = }')
+                            # s_msg = msg.split(']')[0].split(' ')[-1]
+                            # logger.error(f'{s_msg = }')
+                            # self.tts().queue_speech(M_STRINGS['volume_set'] + s_msg, Priority.HIGH, mode=self._mode())
+                        else:
+                            self.tts().queue_speech(msg, Priority.HIGH, mode=self._mode())
+            except AttributeError:
+                pass
+
             current_player_id = self._current_player_id()
             active_player_id = self._active_player_id()
 
@@ -507,6 +535,11 @@ class Log():
                 ''' start normal execution '''
                 if msg:
                     logger.error(f'\nself.msg\n{self.msg}\nmsg\n{msg}')
+                    # this is a hack...
+                    if msg.startswith(M_STRINGS['muted'] + ' ' + M_STRINGS['playing_']) and \
+                            self.msg.startswith(M_STRINGS['title_']):
+                        msg = M_STRINGS['muted'] + ' ' + self.msg
+                        logger.error('replacing msg with {msg}')
                 if msg and self._player_stopped:
                     ''' Refuse to print anything if "Playback stopped"
                         was the last message printed
@@ -559,39 +592,27 @@ class Log():
                 ''' update main message '''
                 # logger.error('*** self.msg = "{}"'.format(self.msg))
                 if self.msg is not None and self.msg != '':
-                    # if self._x_start == 1:
-                    #     self.cursesScreen.erase()
-                    d_msg = ''
-                    if self._x_start == 1:
-                        self.cursesScreen.addstr(0, 0, ' ')
-                    else:
+                    try:
+                        d_msg = fix_chars(self.msg.strip())
+
+                        # Account for: space at start + 2 empty chars at end
+                        available_width = self.width - self._x_start - 1
+
+                        if available_width <= 0:
+                            return
+
+                        # Write the space for visual separation
                         self.cursesScreen.addstr(0, self._x_start-1, ' ')
-                    try:
-                        d_msg = fix_chars(self.msg.strip()[0: self.width])
-                        # logger.error('printing "{}" at x = {}'.format(d_msg, self._x_start))
-                        dd_msg = d_msg.strip()[:self.width -2]
-                        while cjklen(dd_msg) > self.width -2:
-                            dd_msg = dd_msg[:-1]
+
+                        # Write message with safe margins
+                        dd_msg, remaining = cjkslices(d_msg, available_width)
                         self.cursesScreen.addstr(0, self._x_start, dd_msg)
-                    except Exception as e:
-                        logger.error(e)
-                        try:
-                            if self._the_time is None:
-                                d_msg = fix_chars(self.msg.encode('utf-8', 'replace').strip()[0: self.width])
-                            else:
-                                d_msg = fix_chars(self.msg.encode('utf-8', 'replace').strip()[0: self.width - len(self._the_time) - 3])
-                            logger.error(f'printing "{d_msg}" at {self._x_start}')
-                            self.cursesScreen.addstr(0, self._x_start, d_msg)
-                        except:
-                            pass
-                            # logger.error('\n\n\n1 except\n{}\n\n\n'.format(msg))
-                            # if logger.isEnabledFor(logging.ERROR):
-                            #     logger.error('Error updating the Status Bar')
-                    try:
+
+                        # clrtoeol is generally safe - no need for nested try/except
                         self.cursesScreen.clrtoeol()
-                    except:
-                        logger.error('\n\n\n2 except\n\n\n')
-                        pass
+
+                    except Exception as e:
+                        logger.error(f"Status bar update error: {e}")
 
                     if msg:
                         if msg.startswith(M_STRINGS['vol_']) or msg.startswith(M_STRINGS['muted']):
@@ -653,11 +674,14 @@ class Log():
                             d_msg + ' ')
                     except:
                         pass
-                    self.cursesScreen.chgat(
-                        0, self._active_width - cjklen(d_msg) + 1,
-                        cjklen(d_msg) - 1,
-                        curses.color_pair(1)
-                    )
+                    try:
+                        self.cursesScreen.chgat(
+                            0, self._active_width - cjklen(d_msg) + 1,
+                            cjklen(d_msg) - 1,
+                            curses.color_pair(1)
+                        )
+                    except:
+                        pass
                     first_print = self._do_i_print_last_char(first_print)
                     self.cursesScreen.refresh()
                     self._active_width -= cjklen(d_msg)
@@ -773,26 +797,64 @@ class Log():
                         with self._song_title_lock:
                             self._song_title = old_song_title
 
-
     def _get_icon_path(self):
         self.icon_path = self._cnf.notification_image_file
+
         if self.icon_path is None:
+
+            # Platform-specific candidates + package resource target name
             if platform_system().lower().startswith('win'):
-                the_path = (
-                    join(getenv('APPDATA'), 'pyradio', 'help', 'pyradio.ico'),
-                    join(dirname(__file__), 'icons', 'pyradio.ico')
-                )
+                fs_candidates = [
+                    Path(getenv("APPDATA") or "") / "pyradio" / "help" / "pyradio.ico"
+                ]
+                pkg_resource = files("pyradio").joinpath("icons", "pyradio.ico")
+                cache_name = "pyradio.ico"
             else:
-                the_path = (
-                    join(self._cnf.data_dir, 'pyradio.png'),
-                    join(dirname(__file__), 'icons', 'pyradio.png'),
-                    '/usr/share/icons/pyradio.png',
-                    '/usr/local/share/icons/pyradio.png'
-                )
-            for n in the_path:
-                if exists(n):
-                    self.icon_path = n
-                    break
+                fs_candidates = [
+                    Path(self._cnf.data_dir) / "pyradio.png",
+                    Path("/usr/share/icons/pyradio.png"),
+                    Path("/usr/local/share/icons/pyradio.png"),
+                ]
+                pkg_resource = files("pyradio").joinpath("icons", "pyradio.png")
+                cache_name = "pyradio.png"
+
+            # 1) Check filesystem candidates first (user/system)
+            chosen = None
+            for p in fs_candidates:
+                try:
+                    if p.exists():
+                        chosen = str(p)
+                        if logger.isEnabledFor(logging.DEBUG):
+                            logger.debug(f"Found icon on filesystem: {chosen}")
+                        break
+                except Exception:
+                    # ignore permission or other err while probing
+                    continue
+
+            # 2) If not found, try packaged resource -> extract to cache and copy persistently
+            if not chosen:
+                cache_dir = Path(self._cnf.cache_dir)
+                cache_dir.mkdir(parents=True, exist_ok=True)
+
+                if pkg_resource.is_file():
+                    # Resource is a filesystem file
+                    # Use its path directly
+                    chosen = str(pkg_resource)
+                    if logger.isEnabledFor(logging.DEBUG):
+                        logger.debug(f"Using packaged icon directly: {chosen}")
+                else:
+                    # Resource is in a zip/wheel → needs extraction
+                    with as_file(pkg_resource) as tmp_path:
+                        cache_file = cache_dir / cache_name
+                        if not cache_file.exists():
+                            copy2(tmp_path, cache_file)
+                            if logger.isEnabledFor(logging.DEBUG):
+                                logger.debug(f"Copied packaged icon to cache: {cache_file}")
+                        chosen = str(cache_file)
+
+            # 3) Final assignment (may still be None)
+            self.icon_path = chosen
+
         self._repeat_notification.icon_path = self.icon_path
 
     def _show_notification(self, msg):
