@@ -13,6 +13,11 @@ from platform import system
 import re
 import glob
 from urllib.parse import urlparse, unquote
+try:
+    from importlib.resources import files, as_file   # 3.9+
+except ImportError:
+    from importlib_resources import files, as_file   # backport για 3.7–3.8
+from pathlib import Path
 
 from .radio import PyRadio
 from .config import PyRadioConfig
@@ -25,7 +30,11 @@ from .schedule import PyRadioScheduleList
 from .install import get_a_linux_resource_opener
 from .html_help import is_graphical_environment_running
 from .client import client
-from .m3u import list_to_m3u, parse_m3u
+HAS_CHARSET_NORMALIZER = True
+try:
+    from .m3u import list_to_m3u, parse_m3u
+except ImportError:
+    HAS_CHARSET_NORMALIZER = False
 
 try:
     # Windows
@@ -217,26 +226,6 @@ def _win_python_3_12():
             setattr(curses, key, value)
     return stdscr
 
-def create_systemd_service_files():
-    file_names = ('start-headless-pyradio.sh', 'stop-headless-pyradio.sh', 'pyradio.service')
-    if program == 'tmux':
-        files = (
-r'''#!|SHELL|
-touch |HOME|/pyradio.log
-|PROGRAM| new-session -dA -s pyradio-session |PYRADIO| --headless auto
-''',
-r'''#!|SHELL|
-[ -z "$(|PROGRAM| ls | grep pyradio-session)" ] || |PROGRAM| send-keys -t pyradio-session q
-sleep 2
-[ -z "$(|PROGRAM| ls | grep pyradio-session)" ] || |PROGRAM| sennd-keys -t pyradio-session q
-[ -e |HOME|/.config/pyradio/data/server-headless.txt ] && rm |HOME|/.config/pyradio/data/server-headless.txt
-[ -e |HOME|/.local/state/pyradio/server-headless.txt ] && rm |HOME|/.local/state/pyradio/server-headless.txt
-'''
-                )
-        pass
-    else:
-        pass
-
 def shell():
     if sys.version_info[0] == 2 or \
             (sys.version_info.major == 3 and sys.version_info < (3, 7)):
@@ -502,6 +491,9 @@ If nothing else works, try the following command:
 
         # handle playilst validation
         if args.validate:
+            if not HAS_CHARSET_NORMALIZER:
+                print_module_not_found_error('charset-normalizer')
+                sys.exit(1)
             # determine mode (mark or drop)
             mode = args.validate
 
@@ -529,6 +521,9 @@ If nothing else works, try the following command:
             sys.exit(0)
 
         if args.convert:
+            if not HAS_CHARSET_NORMALIZER:
+                print_module_not_found_error('charset-normalizer')
+                sys.exit(1)
             # Determine conversion direction and validate input file
             csv_to_m3u = False
             in_file = args.convert  # Let's say args.convert is "reversed"
@@ -595,23 +590,17 @@ If nothing else works, try the following command:
 
         if not system().lower().startswith('darwin') and \
                 not system().lower().startswith('win'):
-            # if args.generate_systemd_service_files:
-            #     create_systemd_service_files()
-            #     sys.exit()
-            # elif args.terminal:
 
             if args.terminal:
                 import subprocess
                 r = None
-                script = None
-                if script is None:
-                    package_file = path.join(path.dirname(__file__), 'scripts', 'fix_pyradio_desktop_file')
-                    script = path.join(pyradio_config.cache_dir, 'fix_pyradio_desktop_file')
-                    try:
-                        shutil.copy(package_file, script)
-                    except Exception as e:
-                        print(f'Error copying file: "{e}"')
-                        sys.exit(1)
+                script = Path(pyradio_config.cache_dir) / 'fix_pyradio_desktop_file'
+                try:
+                    with as_file(files("pyradio").joinpath("scripts", "fix_pyradio_desktop_file")) as src:
+                        shutil.copy(src, script)
+                except Exception as e:
+                    print(f'Error copying file: {e}')
+                    sys.exit(1)
                     # try:
                     #     from urllib.request import urlretrieve
                     # except:
@@ -673,9 +662,11 @@ If nothing else works, try the following command:
             return
 
         if args.print_config:
-            cnf = path.join(path.dirname(__file__), 'config')
-            with open(cnf, 'r', encoding='utf-8') as f:
-                lines = f.readlines()
+            res = files("pyradio").joinpath("config")
+            with as_file(res) as cnf_path:
+                cnf_path = Path(cnf_path)
+                with cnf_path.open('r', encoding='utf-8') as f:
+                    lines = f.readlines()
             if not pyradio_config.show_no_themes_message:
                 lines.append('')
                 lines.append('# User option (response to a message window)')
@@ -877,7 +868,12 @@ that window to complete the update process.''')
             print(f'      Data dir: "[red]{pyradio_config.data_dir}[/red]"')
             print(f'     State dir: "[red]{pyradio_config.state_dir}[/red]"')
             print(f'     Cache dir: "[red]{pyradio_config.cache_dir}[/red]"')
-            print(f'      Code dir: "[red]{path.dirname(__file__)}[/red]"')
+            code_dir = files("pyradio")
+            if code_dir.isdir():
+                print(f'      Code dir: "[red]{str(code_dir)}[/red]"')
+            else:
+                container = str(code_dir.locate().parent)
+                print(f'      Code dir (container): "[red]{container}[/red]"')
             print(f'Recordings dir: "[red]{pyradio_config.recording_dir}[/red]"')
             return
 
@@ -1142,6 +1138,7 @@ that window to complete the update process.''')
             force_update=args.force_update,
             record=args.record
         )
+
         ''' Setting ESCAPE key delay to 25ms
             Refer to: https://stackoverflow.com/questions/27372068/why-does-the-escape-key-have-a-delay-in-python-curses
         '''
@@ -1253,6 +1250,7 @@ def read_config(pyradio_config):
     elif ret == -2:
         print(f'Config file is malformed: "[red]{pyradio_config.config_file}[/red]"')
         sys.exit(1)
+
     # for n in pyradio_config.opts.keys():
     #     print('{0}: {1}'.format(n, pyradio_config.opts[n]))
     if pyradio_config.xdg_compliant:
@@ -1279,6 +1277,18 @@ def print_simple_error(msg):
     msg = msg.replace('Error: ', '[red]Error: [/red]').replace('PyRadio', '[magenta]PyRadio[/magenta]')
     print(msg)
 
+def print_module_not_found_error(the_module):
+    msg = '''Error: Module [green]MODULE[/green] not found!
+
+Please use your distro package manager to install it
+  (named [green]python-MODULE[/green] or [green]python3-MODULE[/green]),
+  or execute:
+
+    [green]pip install MODULE[/green]
+
+'''.replace('MODULE', the_module)
+    print_simple_error(msg)
+
 def print_playlist_selection_error(a_selection, cnf, ret, exit_if_malformed=True):
     if exit_if_malformed:
         if ret == -1:
