@@ -3,11 +3,20 @@ import locale
 import curses
 import logging
 import glob
-from os import path, sep as dir_sep, access, R_OK
+from os import path, sep as dir_sep, access, R_OK, W_OK
 from shutil import copyfile
 from copy import deepcopy
 from math import sqrt
 import colorsys
+from pathlib import Path
+try:
+    # Python ≥ 3.9
+    from importlib.resources import files, as_file
+    from importlib.resources.abc import Traversable
+except ImportError:
+    # Python 3.7 & 3.8 (backport)
+    from importlib_resources import files, as_file
+    from importlib_resources.abc import Traversable
 from .common import rgb_to_curses_rgb, rgb_to_hex, hex_to_rgb
 from .keyboard import kbkey, check_localized, remove_l10n_from_global_functions
 
@@ -28,7 +37,9 @@ def compare_color_pairs(pair1, pair2, curses_colors):
         return True  # The color pairs are identical
     return False  # The color pairs are different
 
-def is_light_or_dark(rgb_color=[0,128,255]):
+def is_light_or_dark(rgb_color=None):
+    if rgb_color is None:
+        rgb_color = [0, 128, 255]
     [r,g,b]=rgb_color
     """
     https://stackoverflow.com/questions/22603510/is-this-possible-to-detect-a-colour-is-a-light-or-dark-colour
@@ -440,11 +451,12 @@ class PyRadioTheme():
                 return False, f'Error writing theme file: "{out_theme_name}"'
         else:
             ''' copy theme file '''
-            in_file = path.join(path.dirname(__file__), 'themes', theme_name + '.pyradio-theme')
+            in_resource = files("pyradio.themes").joinpath(f"{theme_name}.pyradio-theme")
             try:
-                copyfile(in_file, out_file)
+                with as_file(in_resource) as real_path:
+                    copyfile(real_path, out_file)
                 return True, f'Theme created: "{out_theme_name}"'
-            except:
+            except Exception:
                 return False, f'Error creating file for theme: "{out_theme_name}"'
 
     def open_theme(self, a_theme='', a_path='', print_errors=None, no_curses=False):
@@ -481,7 +493,7 @@ class PyRadioTheme():
             self._colors['Name'] = 'dark_16_colors'
             self._colors['Path'] = ''
             self.applied_theme_name = 'dark_16_colors'
-            self._colors['data'] = {1: (255, 255, 255), 2: (128, 128, 128), 3: (0, 255, 0), 8: (0, 0, 0), 9: (0, 255, 0), 4: (0, 0, 0), 5: (255, 0, 255), 6: (0, 0, 0), 7: (0, 255, 0), 12: (0, 255, 255), 11: (0, 0, 255), 10: (255, 255, 0), 13: (255, 255, 255), 13: (128, 128, 128), 15: (154, 154, 154)}
+            self._colors['data'] = {1: (255, 255, 255), 2: (128, 128, 128), 3: (0, 255, 0), 8: (0, 0, 0), 9: (0, 255, 0), 4: (0, 0, 0), 5: (255, 0, 255), 6: (0, 0, 0), 7: (0, 255, 0), 12: (0, 255, 255), 11: (0, 0, 255), 10: (255, 255, 0), 13: (255, 255, 255), 14: (128, 128, 128), 15: (154, 154, 154)}
             if not no_curses:
                 if curses.COLORS > 16:
                     self._colors['data'][16] = (255, 255, 255)
@@ -587,17 +599,65 @@ class PyRadioTheme():
         return ret
 
     def _get_theme_path(self, a_theme):
-        #self.root_path = path.join(path.dirname(__file__), 'stations.csv')
-        theme_dirs = [path.join(self._cnf.stations_dir, 'themes'),
-                      path.join(path.dirname(__file__), 'themes')]
-        for theme_dir in theme_dirs:
-            files = glob.glob(path.join(theme_dir, '*.pyradio-theme'))
-            if files:
-                for a_file in files:
-                    a_theme_name = a_file.split(dir_sep)[-1].replace('.pyradio-theme', '')
-                    if a_theme_name == a_theme:
-                        return a_file
-        return ''
+        """
+        Locate a theme file by name, searching both user and package directories.
+
+        1. First search the user themes directory (real filesystem path).
+        2. Then search the package's internal themes directory (may be inside a zip/egg).
+        3. If found inside the package resources, extract it to the cache directory and return the real path.
+
+        Parameters
+        ----------
+        a_theme : str
+            The name of the theme (without extension).
+
+        Returns
+        -------
+        str
+            Absolute path to the located theme file, or an empty string if not found.
+        """
+        # logger.error(f'{a_theme = }')
+        try:
+            # Python ≥ 3.9
+            from importlib.resources import files, as_file
+            from importlib.resources.abc import Traversable
+        except ImportError:
+            # Python 3.7 & 3.8 (backport)
+            from importlib_resources import files, as_file
+            from importlib_resources.abc import Traversable
+
+        user_theme_dir = Path(self._cnf.stations_dir) / 'themes'
+        package_theme_dir = files("pyradio") / "themes"
+        cache_dir = Path(self._cnf.cache_dir) / 'themes'
+
+        # List of theme directories: user path first, package resources second
+        theme_dirs_to_search = [
+            (user_theme_dir, False),    # (dir, is_package_resource)
+            (package_theme_dir, True)
+        ]
+        # logger.error(f'{theme_dirs = }')
+
+
+        # Search in user dir firts
+        for theme_dir, is_package_resource in theme_dirs_to_search:
+            # Case 1: It is Traversable (package source)
+            if is_package_resource and isinstance(theme_dir, Traversable):
+                for res in theme_dir.iterdir():
+                    if res.name == f"{a_theme}.pyradio-theme":
+                        # Copy to cache if a package source
+                        cache_dir.mkdir(parents=True, exist_ok=True)
+                        cached_theme_path = cache_dir / res.name
+                        with as_file(res) as tmp_path:
+                            copyfile(tmp_path, cached_theme_path)
+                        return str(cached_theme_path)
+            # Case 2: It is a normal system path
+            elif isinstance(theme_dir, Path) and theme_dir.is_dir():
+                theme_path = theme_dir / f"{a_theme}.pyradio-theme"
+                if theme_path.is_file():
+                    return str(theme_path)
+
+        # If we got this far, nothing found
+        return None
 
 
 class PyRadioThemeReadWrite():
@@ -690,7 +750,7 @@ class PyRadioThemeReadWrite():
             # logger.error('sp = {}'.format(sp))
             # logger.error('names = {}'.format(names))
             names[sp[0].strip()] = sp[1:]
-            for k, v in names.items():
+            for _, v in names.items():
                 for n in (0, 1):
                     try:
                         v[n] = v[n].strip()
@@ -1004,43 +1064,78 @@ class PyRadioThemeSelector():
         if self._first_theme_to_watch == 0:
             self._first_theme_to_watch = len(self._themes)
 
-        for a_theme in self._themes:
-            self._max_title_width = max(len(theme[0]) for theme in self._themes)
+        self._max_title_width = max(len(theme[0]) for theme in self._themes)
 
         if self.log:
             self.log(f'max_title_width = {self._max_title_width}\n')
         self._get_config_and_applied_theme(touch_selection)
         self._get_metrics()
 
-    def _scan_for_theme_files(self, cnf_path, user_themes_first=False):
+    def _scan_for_theme_files(self, cnf_path, user_themes_first: bool = False):
+        """
+        Scan for available theme files.
+
+        This function collects theme names and their corresponding paths from:
+          1. User themes directory (real filesystem path).
+          2. Package internal themes directory (may be zipped).
+
+        If a theme comes from the package, its path is set to an empty string (''),
+        signaling that it must be extracted/resolved later by _get_theme_path().
+
+        Parameters
+        ----------
+        cnf_path : str
+            Path to the user's configuration directory.
+        user_themes_first : bool, optional
+            If True, list user themes first.
+
+        Returns
+        -------
+        list[list[str, str]]
+            A list of [theme_name, theme_path] pairs. Some entries are headers
+            of the form [header_text, '-'].
+        """
         out_themes = []
-        #self.root_path = path.join(path.dirname(__file__), 'stations.csv')
-        theme_dirs = [path.join(path.dirname(__file__), 'themes'),
-                      path.join(cnf_path, 'themes')]
+
+        # User and package theme directories
+        user_dir = path.join(cnf_path, 'themes')
+        package_dir = files("pyradio").joinpath("themes")
+
+        # Build directory list depending on priority
+        theme_dirs = [(user_dir, 'User Themes'), (package_dir, 'System Themes')]
         if user_themes_first:
             theme_dirs.reverse()
-        for i, theme_dir in enumerate(theme_dirs):
-            files = glob.glob(path.join(theme_dir, '*.pyradio-theme'))
-            if files:
-                tmp_themes = []
-                for a_file in files:
-                    theme_name = a_file.split(dir_sep)[-1].replace('.pyradio-theme', '')
 
+        for theme_dir, header in theme_dirs:
+            tmp_themes = []
+
+            # Case 1: user directory (real path)
+            if isinstance(theme_dir, str) and path.isdir(theme_dir):
+                theme_files = glob.glob(path.join(theme_dir, '*.pyradio-theme'))
+                for a_file in theme_files:
+                    theme_name = path.basename(a_file).replace('.pyradio-theme', '')
                     ret, _ = self._cnf.is_project_theme(theme_name)
-                    if ret is None:
-                        if not self._cnf.is_default_file(theme_name):
-                            tmp_themes.append([theme_name, a_file])
-                if tmp_themes:
-                    tmp_themes.sort()
-                    tmp_themes.reverse()
-                    if i == 0:
-                        tmp_themes.append(['System Themes', '-'])
-                    else:
-                        tmp_themes.append(['User Themes', '-'])
-                    tmp_themes.reverse()
-                    out_themes.extend(tmp_themes)
+                    if ret is None and not self._cnf.is_default_file(theme_name):
+                        tmp_themes.append([theme_name, a_file])
 
-        ''' add auto update themes, if not already there '''
+            # Case 2: package directory (Traversable, may be zipped)
+            elif isinstance(theme_dir, Traversable):
+                for res in theme_dir.iterdir():
+                    if res.name.endswith('.pyradio-theme'):
+                        theme_name = res.name[:-14]
+                        ret, _ = self._cnf.is_project_theme(theme_name)
+                        if ret is None and not self._cnf.is_default_file(theme_name):
+                            # '' indicates package resource, to be resolved later
+                            tmp_themes.append([theme_name, ''])
+
+            if tmp_themes:
+                tmp_themes.sort()
+                tmp_themes.reverse()
+                tmp_themes.append([header, '-'])
+                tmp_themes.reverse()
+                out_themes.extend(tmp_themes)
+
+        # Add auto update themes, if not already there
         tmp_themes = []
         for n in self._cnf.auto_update_frameworks:
             if n.can_auto_update:
@@ -1052,6 +1147,7 @@ class PyRadioThemeSelector():
             tmp_themes.append(['Ext. Themes Projects', '-'])
             tmp_themes.reverse()
             out_themes.extend(tmp_themes)
+
         return out_themes
 
     def _theme_name_in_themes(self, themes_list, theme_name):
@@ -1274,10 +1370,11 @@ class PyRadioThemeSelector():
             else:
                 self._win.addstr(i+1, 1, token + self._themes[an_item][0], col)
 
-        try:
-            self._win.move(sel, self._width - 2)
-        except:
-            pass
+        # Leftover from a bad copy/paste?
+        # try:
+        #     self._win.move(sel, self._width - 2)
+        # except:
+        #     pass
         ''' display transparency indicator '''
         if not self.changed_from_config:
             self._win.addstr(self._height-1, self._width - 4, '[T]', curses.color_pair(self._box_color_pair))
@@ -1331,18 +1428,34 @@ class PyRadioThemeSelector():
         self.selection = len(self._themes)
 
     def _is_theme_read_only(self, theme_path):
-        if theme_path:
-            themes_path = path.join(path.dirname(__file__), 'themes')
-            if themes_path == path.dirname(theme_path):
-                return True
-            else:
-                if access(theme_path, R_OK):
-                    return False
-                else:
-                    return True
-        else:
+        """
+        Determine if a theme is read-only.
+
+        Read-only if:
+          1. No real path ('' or inside zip/wheel)
+          2. Located in cache_dir (even if writable)
+          3. Located in system dir and not writable
+        """
+        # Case 1: zip/wheel resource (no real file)
+        if not theme_path:
             return True
 
+        # Normalize
+        theme_dir = path.dirname(theme_path)
+
+        # Case 2: cache_dir copy (always read-only)
+        if theme_dir.startswith(self._cnf.cache_dir):
+            return True
+
+        # Case 3: system dir (package installed themes)
+        # it is ok, zip/wheel taken care of
+        system_themes_path = path.join(path.dirname(__file__), 'themes')
+        if theme_dir == system_themes_path:
+            return True
+
+        # Otherwise, check real FS permissions (user themes)
+        return not access(theme_path, R_OK | W_OK)
+
     def keypress(self, char):
         """ PyRadioThemeSelector keypress
             returns theme_id, save_theme
